/**
 * Copyright ProjectNya ( http://wonderfl.net/user/ProjectNya )
 * MIT License ( http://www.opensource.org/licenses/mit-license.php )
 * Downloaded from: http://wonderfl.net/c/wqVz
 */

////////////////////////////////////////////////////////////////////////////////
// QuickBox2D (1)
//
// [AS3.0] QuickBox2Dを試すのだ！ (12)
// http://www.project-nya.jp/modules/weblog/details.php?blog_id=1428
////////////////////////////////////////////////////////////////////////////////

package {

    import flash.display.Sprite;
    import flash.display.MovieClip;
    import flash.geom.Rectangle;
    import flash.events.MouseEvent;
    import flash.utils.Dictionary;
    import flash.geom.Matrix;
    import flash.display.GradientType;
    import flash.display.SpreadMethod;
    import flash.display.InterpolationMethod;
    import com.actionsnippet.qbox.QuickBox2D;
    import com.actionsnippet.qbox.QuickObject;

    [SWF(backgroundColor="#FFFFFF", width="465", height="465", frameRate="30")]

    public class Main extends Sprite {
        private var container:MovieClip;
        private var world:QuickBox2D;
        private var circles:Array;
        private var dictionary:Dictionary;
        private static var max:uint = 50;
        private var center:QuickObject;
        private var playBtn:Btn;
        private var clearBtn:Btn;
        private var gravitationBtn:Btn;
        private var mode:Boolean = false;
        private var txt:Label;

        public function Main() {
            //Wonderfl.capture_delay(1);
            init();
        }

        private function init():void {
            draw();
            //
            var label:Label = new Label(200, 24, 20, Label.CENTER);
            addChild(label);
            label.x = 132;
            label.y = 185;
            label.textColor = 0xFFFFFF;
            label.text = "QuickBox2D";
            label.alpha = 0.6;
            txt = new Label(200, 16, 16, Label.CENTER);
            addChild(txt);
            txt.x = 132;
            txt.y = 215;
            txt.textColor = 0xFFFFFF;
            txt.text = "gravitation";
            txt.alpha = 0.6;
            //
            container = new MovieClip();
            addChild(container);
            world = new QuickBox2D(container, {renderJoints: false});
            createStageWalls(new Rectangle(0, 0, 465, 400));
            center = world.addCircle({x: 7.75, y: 7, radius: 3, density: 0, lineAlpha: 0.1, fillAlpha: 0, restitution: 0.4, isBullet: false});
            world.setDefault({lineAlpha: 0, fillColor: 0xFFFFFF, fillAlpha: 0.6, density: 0.1, friction: 1, restitution: 0.4, linearDamping: 1, angularDamping: 2});
            circles = new Array();
            dictionary = new Dictionary(true);
            addCircle(5);
            world.start();
            //
            playBtn = new Btn();
            addChild(playBtn);
            playBtn.x = 132;
            playBtn.y = 450;
            playBtn.init({label: "play"});
            playBtn.addEventListener(MouseEvent.CLICK, addCircles, false, 0, true);
            clearBtn = new Btn();
            addChild(clearBtn);
            clearBtn.x = 332;
            clearBtn.y = 450;
            clearBtn.init({label: "clear"});
            clearBtn.addEventListener(MouseEvent.CLICK, clearCircles, false, 0, true);
            //
            gravitationBtn = new Btn();
            addChild(gravitationBtn);
            gravitationBtn.x = 232;
            gravitationBtn.y = 450;
            gravitationBtn.init({label: "gravitation", width: 100});
            gravitationBtn.addEventListener(MouseEvent.CLICK, gravitation, false, 0, true);
        }
        private function addCircles(evt:MouseEvent):void {
            addCircle(5);
        }
        private function clearCircles(evt:MouseEvent):void {
            removeCircles(circles.length);
            removeJoints();
            circles = new Array();
            dictionary = new Dictionary(true);
            addCircle(5);
        }
        private function addCircle(unit:uint = 1):void {
            if (circles.length >= max) removeCircles(unit);
            for (var n:uint = 0; n < unit; n++) {
                var px:Number = Math.random()*13.5 + 1;
                var circle:QuickObject = world.addCircle({x: px, y: -2, radius: 1/2, isBullet: false});
                circles.push(circle);
                if (mode) addJoint(circle);
            }
        }
        private function removeCircles(unit:uint):void {
            for (var n:uint = 0; n < unit; n++) {
                var circle:QuickObject = circles[n];
                circle.destroy();
                circle = null;
            }
            circles.splice(0, unit);
        }
        private function gravitation(evt:MouseEvent):void {
            mode = !mode;
            if (mode) {
                txt.alpha = 0.9;
                addJoints();
            } else {
                txt.alpha = 0.6;
                removeJoints();
            }
        }
        private function addJoints():void {
            for (var n:uint = 0; n < circles.length; n++) {
                var circle:QuickObject = circles[n];
                addJoint(circle);
            }
        }
        private function removeJoints():void {
            for (var n:uint = 0; n < circles.length; n++) {
                var circle:QuickObject = circles[n];
                removeJoint(circle);
            }
        }
        private function addJoint(circle:QuickObject):void {
            var joint:QuickObject = world.addJoint({a: circle.body, b: center.body, length: 7/2, frequencyHz: 5, dampingRatio: 0.8, collideConnected: true});
            dictionary[circle] = joint;
        }
        private function removeJoint(circle:QuickObject):void {
            var joint:QuickObject = dictionary[circle];
            joint.destroy();
            joint = null;
        }
        private function createStageWalls(area:Rectangle):void {
            var px:Number = area.x/30;
            var py:Number = area.y/30;
            var sw:Number = area.width/30;
            var sh:Number = area.height/30;
            world.addBox({x: px + sw/2, y: py + sh + 0.5, width: sw, height: 1, density: 0, lineAlpha: 0, fillAlpha: 0, restitution: 0.8});
            //world.addBox({x: px + sw/2, y: py - 0.5, width: sw, height: 1, density: 0, lineAlpha: 0, fillAlpha: 0});
            world.addBox({x: px - 0.5, y: py + sh/2, width: 1, height: sh + 1, density: 0, lineAlpha: 0, fillAlpha: 0});
            world.addBox({x: px + sw + 0.5, y: py + sh/2, width: 1, height: sh + 1, density: 0, lineAlpha: 0, fillAlpha: 0});
        }
        private function draw():void {
            drawSky();
            drawGrond();
        }
        private function drawSky():void {
            var colors:Array = [0x00AAE4, 0x0069A0];
            var alphas:Array = [1, 1];
            var ratios:Array = [0, 255];
            var matrix:Matrix = new Matrix();
            matrix.createGradientBox(465, 375, Math.PI*1.5, 0, 0);
            graphics.beginGradientFill(GradientType.LINEAR, colors, alphas, ratios, matrix, SpreadMethod.PAD, InterpolationMethod.RGB, 0);
            graphics.drawRect(0, 0, 465, 375);
            graphics.endFill();
        }
        private function drawGrond():void {
            var colors:Array = [0x99CC33, 0x7EB133];
            var alphas:Array = [1, 1];
            var ratios:Array = [0, 255];
            var matrix:Matrix = new Matrix();
            matrix.createGradientBox(465, 60, 0.5*Math.PI, 0, 0);
            graphics.beginGradientFill(GradientType.LINEAR, colors, alphas, ratios, matrix);
            graphics.drawRect(0, 375, 465, 60);
            graphics.endFill();
        }
        
    }

}


//////////////////////////////////////////////////
// Btnクラス
//////////////////////////////////////////////////

import flash.display.Sprite;
import flash.display.Shape;
import flash.text.TextField;
import flash.text.TextFieldType;
import flash.text.AntiAliasType;
import flash.text.TextFormat;
import flash.text.TextFormatAlign;
import flash.filters.GlowFilter;
import flash.events.MouseEvent;

class Btn extends Sprite {
    public var id:uint;
    private var shade:Shape;
    private var bottom:Shape;
    private var light:Shape;
    private var base:Shape;
    private var txt:TextField;
    private var label:String = "";
    private static var fontType:String = "_ゴシック";
    private var _width:uint = 60;
    private static var _height:uint = 20;
    private static var corner:uint = 5;
    private var type:uint = 1;
    private static var bColor:uint = 0xFFFFFF;
    private static var sColor:uint = 0x000000;
    private static var upColor:uint = 0x666666;
    private static var overColor:uint = 0x333333;
    private static var offColor:uint = 0x999999;
    private static var gColor:uint = 0x0099FF;
    private var blueGlow:GlowFilter;
    private var shadeGlow:GlowFilter;
    private var _clicked:Boolean = false;
    private var _enabled:Boolean = true;

    public function Btn() {
    }

    public function init(option:Object):void {
        if (option.id != undefined) id = option.id;
        if (option.label != undefined) label = option.label;
        if (option.width != undefined) _width = option.width;
        if (option.type != undefined) type = option.type;
        draw();
    }
    private function draw():void {
        switch (type) {
        case 1 :
            bColor = 0xFFFFFF;
            sColor = 0x000000;
            upColor = 0x666666;
            overColor = 0x333333;
            offColor = 0x999999;
            break;
        case 2 :
            bColor = 0x000000;
            sColor = 0xFFFFFF;
            upColor = 0x666666;
            overColor = 0x999999;
            offColor = 0x333333;
            break;
        }
        blueGlow = new GlowFilter(gColor, 0.6, 5, 5, 2, 3, false, true);
        shadeGlow = new GlowFilter(sColor, 0.3, 4, 4, 2, 3, false, true);
        shade = new Shape();
        bottom = new Shape();
        light = new Shape();
        base = new Shape();
        txt = new TextField();
        addChild(shade);
        addChild(bottom);
        addChild(light);
        addChild(base);
        addChild(txt);
        createBase(shade, _width, _height, corner, sColor);
        shade.filters = [shadeGlow];
        createBase(bottom, _width, _height, corner, sColor, 0.3);
        createBase(light, _width, _height, corner, gColor);
        light.filters = [blueGlow];
        createBase(base, _width, _height, corner, bColor);
        txt.x = -_width*0.5;
        txt.y = -_height*0.5;
        txt.width = _width;
        txt.height = _height - 1;
        txt.type = TextFieldType.DYNAMIC;
        txt.selectable = false;
        //txt.embedFonts = true;
        //txt.antiAliasType = AntiAliasType.ADVANCED;
        var tf:TextFormat = new TextFormat();
        tf.font = fontType;
        tf.size = 12;
        tf.align = TextFormatAlign.CENTER;
        txt.defaultTextFormat = tf;
        txt.text = label;
        enabled = true;
        mouseChildren = false;
    }
    private function rollOver(evt:MouseEvent):void {
        _over();
    }
    private function rollOut(evt:MouseEvent):void {
        _up();
    }
    private function press(evt:MouseEvent):void {
        _down();
    }
    private function release(evt:MouseEvent):void {
        _up();
    }
    private function click(evt:MouseEvent):void {
    }
    private function _up():void {
        txt.y = -_height*0.5;
        txt.textColor = upColor;
        base.y = -1;
        light.visible = false;
        light.y = -1;
    }
    private function _over():void {
        txt.y = -_height*0.5;
        txt.textColor = overColor;
        base.y = -1;
        light.visible = true;
        light.y = -1;
    }
    private function _down():void {
        txt.y = -_height*0.5 + 1;
        txt.textColor = overColor;
        base.y = 0;
        light.visible = true;
        light.y = 0;
    }
    private function _off():void {
        txt.y = -_height*0.5 + 1;
        txt.textColor = offColor;
        base.y = 0;
        light.visible = false;
        light.y = 0;
    }
    public function get clicked():Boolean {
        return _clicked;
    }
    public function set clicked(param:Boolean):void {
        _clicked = param;
        enabled = !_clicked;
        if (_clicked) {
            _down();
        } else {
            _up();
        }
    }
    public function get enabled():Boolean {
        return _enabled;
    }
    public function set enabled(param:Boolean):void {
        _enabled = param;
        buttonMode = _enabled;
        mouseEnabled = _enabled;
        useHandCursor = _enabled;
        if (_enabled) {
            _up();
            addEventListener(MouseEvent.MOUSE_OVER, rollOver, false, 0, true);
            addEventListener(MouseEvent.MOUSE_OUT, rollOut, false, 0, true);
            addEventListener(MouseEvent.MOUSE_DOWN, press, false, 0, true);
            addEventListener(MouseEvent.MOUSE_UP, release, false, 0, true);
            addEventListener(MouseEvent.CLICK, click, false, 0, true);
        } else {
            _off();
            removeEventListener(MouseEvent.MOUSE_OVER, rollOver);
            removeEventListener(MouseEvent.MOUSE_OUT, rollOut);
            removeEventListener(MouseEvent.MOUSE_DOWN, press);
            removeEventListener(MouseEvent.MOUSE_UP, release);
            removeEventListener(MouseEvent.CLICK, click);
        }
    }
    private function createBase(target:Shape, w:uint, h:uint, c:uint, color:uint, alpha:Number = 1):void {
        target.graphics.beginFill(color, alpha);
        target.graphics.drawRoundRect(-w*0.5, -h*0.5, w, h, c*2);
        target.graphics.endFill();
    }

}


//////////////////////////////////////////////////
// Labelクラス
//////////////////////////////////////////////////

import flash.display.Sprite;
import flash.text.TextField;
import flash.text.TextFieldType;
import flash.text.TextFieldAutoSize;
import flash.text.AntiAliasType;
import flash.text.TextFormat;
import flash.text.TextFormatAlign;

class Label extends Sprite {
    private var txt:TextField;
    private static var fontType:String = "_ゴシック";
    private var _width:uint = 20;
    private var _height:uint = 20;
    private var size:uint = 12;
    public static const LEFT:String = TextFormatAlign.LEFT;
    public static const CENTER:String = TextFormatAlign.CENTER;
    public static const RIGHT:String = TextFormatAlign.RIGHT;

    public function Label(w:uint, h:uint, s:uint = 12, align:String = LEFT) {
        _width = w;
        _height = h;
        size = s;
        draw(align);
    }

    private function draw(align:String):void {
        txt = new TextField();
        addChild(txt);
        txt.width = _width;
        txt.height = _height;
        txt.autoSize = align;
        txt.type = TextFieldType.DYNAMIC;
        txt.selectable = false;
        //txt.embedFonts = true;
        //txt.antiAliasType = AntiAliasType.ADVANCED;
        var tf:TextFormat = new TextFormat();
        tf.font = fontType;
        tf.size = size;
        tf.align = align;
        txt.defaultTextFormat = tf;
        textColor = 0x000000;
    }
    public function set text(param:String):void {
        txt.text = param;
    }
    public function set textColor(param:uint):void {
        txt.textColor = param;
    }

}
